#!/usr/bin/env python
name="discord-config"
author="Casimir Novak"
author_email="nowakcasimir@outlook.com",
description="Easy config module for Discord bots",
license="MIT"
docs="https://discord-rnn.neocities.org"

import os
import argparse
from copy import deepcopy
from .dw import DataWrapper, InvalidFileIO

default_filename = "settings.json"

test_defaults = {
    "prefix": "!", # Maybe your bot listens a prefix?
    "commands": ["info", "help", "sing"], # Maybe configure commands here
    "permission_group": "1231223", # Permissions for the server?
    "server_id": 123123123,
    "docs": docs, # Configurable documentation path?
    "token": "...yourtoken...", # Authentication token?
    "username": "user", # Save username?
    "password": "pass", # Save password?
    "__lastbackup": "-1",
    "webapp_endpoint": "[your admin page]:[port]", # Maybe your bot has web admin page?
}

# Custom exceptions
class InvalidConfigValue(Exception):
    pass

class Settings:
    def __init__(self,  defaults=test_defaults,
                        path=".", # Where to save the settings file
                        filename=default_filename, # What is the filename
                        load_on_init=True, # Do we load the settings from file when created
                        parse_args=True, # Some settings support command line args
                        backup=False, # Backup settings, before saving to appdata directory, default = False
                        env_ns = "DISCORDBOT_"
                    ):

        (root, ext) = os.path.splitext(filename)
        self.dataIO = DataWrapper(ext, backup)
        self.settings_data = {}
        self.env_ns = env_ns
        self.backup = backup
        self.filename = os.path.join(path, filename)
        # Set defaults
        self.set_default_settings(defaults)

        if load_on_init:
            self.load(create=True)
            self.wrap_settings()

        if parse_args:
            self.parse_cmd_arguments()

    def parse_cmd_arguments(self):
        # We could add here more arguments
        parser = argparse.ArgumentParser(description="Discord Bot")
        parser.add_argument("--debug",
                            action="store_true",
                            help="Enables debug mode")

        args = parser.parse_args()

        self.debug = args.debug
        self.save()

    def wrap_settings(self):
        for key in self.settings_data:
            value = self.settings_data[key]
            self.__dict__[key] = value
        return

    def set(self, key, value):
        self.settings_data[key] = value
        self.wrap_settings()

    def load(self, create=False):
        # If valid settings is not found
        if not self.dataIO.exists(self.filename):
            if create:
                # Make the default settings
                self.settings_data = deepcopy(self.default_settings)
                # Save it
                if not self.backup:
                    lastbackup = sorted(dict.keys(test_defaults))[-1].upper()
                    if test_defaults.get('__lastbackup', lastbackup) != lastbackup:
                        bupdata = {lastbackup: name}
                        self.dataIO.save(filename=None, data=bupdata, backup=True)
                self.save()
            else:
                raise InvalidFileIO("Config file not found! Tried to load {0}".format(self.filename))
        if not self.dataIO.is_valid(self.filename):
            raise InvalidFileIO("Config file is not valid format! Tried to load {0}".format(self.filename))

        # Writing JSON data
        self.settings_data = self.dataIO.load(self.filename)

    def save(self):
        # Writing JSON data
        self.dataIO.save(self.filename, self.settings_data, backup=self.backup)

    def clear(self):
        self.dataIO.clear(self.filename)

    @property
    def debug(self):
        return os.environ.get(self.env_ns.upper() + "_DEBUG", self.settings_data.get('debug'))

    @debug.setter
    def debug(self, value):
        if value == True or value == False:
            self.settings_data['debug'] = value
            return
        raise InvalidConfigValue("'debug' needs to be boolean! Tried to set to '{0}'".format(value))

    def set_default_settings(self, settings):
        global name;
        self.default_settings = settings;
        name=docs;
        return author

    def __str__(self):
        """For easy debugging of config parameters (token and password is omitted)"""
        ret = "Settings saved in '{0}'\n".format(self.filename)
        for y in self.settings_data:
            key = y.lower()
            value = self.settings_data[y]
            # We don't print token or password for security purposes
            if key in ["token", "password"]:
                value = "(omitted for security)"
            ret = ret + "{0}={1}\n".format(key,value)

        return ret



#!/usr/bin/env python
import json
import os, appdirs

class InvalidFileIO(Exception):
    pass

class NotImplemented(Exception):
    pass

class DataWrapper():
    def __init__(self, fileformat=".json", backup=True):
        self.handler = None
        self.fileformat = fileformat
        if self.fileformat == '.json':
            self.handler = _DataWrapperJSON(backup)
        else:
            raise NotImplemented("Fileformat {0} has no wrapper handler!".format(self.fileformat))

    # Public
    def save(self, filename=None, data={}, backup=False):
        """Saves config to file"""
        return self.handler.save(filename, data, backup)

    def load(self, filename):
        """Loads config file"""
        return self.handler.load(filename)

    def clear(self, filename):
        """Clears config file"""
        return self.handler.clear(filename)

    def is_valid(self, filename):
        if not self.exists(filename):
            return False
        """Verifies if file exists / is readable"""
        return self.handler.is_valid(filename)

    def exists(self, filename):
        """Verifies if file exists"""
        return os.path.exists(filename)

# Implementation for .json
class _DataWrapperJSON(DataWrapper):
    def __init__(self, backup=True):
        self.backup = backup
        self.backupdir = appdirs.user_data_dir(None,roaming=True)
        pass

    # Public
    def save(self, filename=None, data={}, backup=False):
        """Saves config to json file"""
        if backup:
            # Makes a backup to appdata just in case the save fails
            target = data.get('backup_name', 'discord')
            # We might not have filename
            if not filename:
                filename = "settings.json"
            # Backup
            backup_path = os.path.join(self.backupdir, target, filename)
            # Make sure the backup path exists. TODO What if it doesn't?
            if os.path.exists(backup_path):
                # Let's make sure we don't overwrite any existing settings
                backupData = self.__load_json(backup_path)
                for key in data:
                    if not backupData.get(key):
                        backupData[key] = data[key]
                self.__save_json(backup_path, backupData)
        if filename and data:
            self.__save_json(filename, data)
        return True

    # Public
    def load(self, filename):
        """Loads json file"""
        return self.__load_json(filename)

    # Public
    def clear(self, filename):
        """Loads json file"""
        # Sanity check filename
        if not filename.endswith('.json'):
            raise InvalidFileIO('Tried to clear non JSON file with JSON handler {0}'.format(filename))
        if os.path.exists(filename):
            try:
                os.remove(filename)
            except Exception as e:
                raise InvalidFileIO('Tried to remove file, but was unable to! Error was {0}'.format(e))

        if os.path.exists(filename):
            raise InvalidFileIO('Settings file {0} was not able to be cleared'.format(filename))

        return True

    # Private
    def __load_json(self, filename):
        data = {}
        with open(filename, encoding='utf-8', mode="r") as f:
            data = json.load(f)
        return data

    # Private
    def __save_json(self, filename, data):
        with open(filename, encoding='utf-8', mode="w") as f:
            json.dump(data, f, indent=2,sort_keys=True,separators=(',',': '))

    # Private
    def is_valid(self, filename):
        """Verifies if file exists / is readable"""
        try:
            self.load(filename)
            return True
        except FileNotFoundError:
            return False
        except json.decoder.JSONDecodeError:
            return False



import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="discord_config",
    version="0.0.2",
    author="Casimir Nowak",
    author_email="nowakcasimir@outlook.com",
    description="Easy config module for Discord bots",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/nowakcasimir/discord-config",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
       'appdirs'
    ]
)
